package org.mtom.samples;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;

import javax.activation.DataHandler;
import javax.ejb.Stateless;
import javax.jws.WebService;
import javax.mail.util.ByteArrayDataSource;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.ws.soap.MTOM;

import org.jboss.ws.api.annotation.WebContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import domain.mtom.schema.mtom.mtomdataservice.v1.GetSomeDataResponseType;
import domain.mtom.services.mtom.mtomdataservice.v1.GetSomeDataFault;
import domain.mtom.services.mtom.mtomdataservice.v1.MTOMDataServicePortType;

@Stateless
@MTOM
@WebService(targetNamespace = "http://mtom.domain/services/mtom/MTOMDataService/v1", 
	portName = "MTOMDataServicePort", 
	serviceName = "MTOMDataService", 
	wsdlLocation = "WEB-INF/wsdl/MTOMDataService.wsdl")
@WebContext(contextRoot="/", urlPattern="/v1", transportGuarantee="CONFIDENTIAL", secureWSDLAccess=false)
public class MTOMServiceImpl implements MTOMDataServicePortType {
	private static final Logger LOG = LoggerFactory.getLogger(MTOMServiceImpl.class);

	public GetSomeDataResponseType getSomeData(
			String input)
			throws GetSomeDataFault {
		GetSomeDataResponseType response = new GetSomeDataResponseType();
		
		final ByteArrayOutputStream baos = new ByteArrayOutputStream();
		GZIPOutputStream gzos = null;
		try {
			final JAXBContext jaxbContext = JAXBContext.newInstance(GetSomeDataResponseType.class);
		    final Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
			
		    gzos = new GZIPOutputStream(baos);
		    gzos.write("somedata".getBytes());
		    gzos.finish();

		    final byte[] compressedByteArray = baos.toByteArray();
		    final ByteArrayDataSource ds = new ByteArrayDataSource(compressedByteArray, "application/octet-stream");
		    final DataHandler dataHandler = new DataHandler(ds);
		    
		    response.setSomeDataArray(dataHandler);
		    
		} catch (final Exception ex) {
			LOG.error("error", ex);
		} finally {
		    try {
		    	baos.close();
		    	if (gzos != null)
		    		gzos.close();
		    } catch (final IOException ioe) {
		    	LOG.error("error", ioe);
		    }
		}	
		
		return response;
	}	
}
